// WStar PinyoDrawing component

//  PROPIX, Written by Pintr Gbor
// Szkesfehrvr, Krivnyi u. 15.
// H-8000, HUNGARY
// Tel: +36 30 9972445
// Fax: +36 22 304326
// Email: propix@mail.datatrans.hu
//        Pinter.Gabor@freemail.hu
// Web: www.propix.hu
//      www.datatrans.hu/propix

// Revisions:
//  V1.0  01/19/1997  First version written in C
//  V3.0  02/19/1998  Rewritten in Delphi 3
//  V5.0  08/25/2000  Recompiled with Delphi 5
//  V5.01 11/14/2000  More features, source reorganized
//  V5.02 11/30/2000  Transparent background added
//  V5.03 12/13/2000  PinyoDrawing is a component

unit DrawUnit;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, ExtCtrls, StdCtrls;

type
  TPicture=(pcStar, pcPascal, pcNephroid);                                      // Drawings
  TPoints= array of TPoint;                                                     // private Point coordinates

  TPinyoDrawing= class(TGraphicControl)
  private
    FPoints: Integer;                                                           // Number of points
    FPicture: TPicture;                                                         // Drawing
    FBackColor: TColor;                                                         // Background color
    FForeColor: TColor;                                                         // Foreground color
    FPenMode: TPenMode;                                                         // Pen write mode
    FPenStyle: TPenStyle;                                                       // Pen style
    FPenWidth: Integer;                                                         // Pen width
    FBrushStyle: TBrushStyle;                                                   // Brush style
    CirclePoints: TPoints;                                                      // Point coordinates
    Center: Tpoint;                                                             // Center coordinates
    Radius: Integer;                                                            // Radius
    procedure SetPoints(Value: Integer);                                        // Set property Points
    procedure SetPicture(Value: TPicture);                                      // Set property Picture
    procedure SetBackColor(Value: TColor);                                      // Set property BackColor
    procedure SetForeColor(Value: TColor);                                      // Set property ForeColor
    procedure SetPenMode(Value: TPenMode);                                      // Set property PenMode
    procedure SetPenStyle(Value: TPenStyle);                                    // Set property PenStyle
    procedure SetPenWidth(Value: Integer);                                      // Set property PenWidth
    procedure SetBrushStyle(Value: TBrushStyle);                                // Set property BrushStyle
    procedure Calculate;                                                        // Recalculate CirclePoints
    procedure WMEraseBkgnd(var msg: TWMEraseBkgnd); message WM_ERASEBKGND;      // Supress background paint
  published
    property Align;                                                             // Alignment
    property Points: Integer read FPoints write SetPoints;                      // Number of points
    property Picture: TPicture read FPicture write SetPicture;                  // Drawing
    property BackColor: TColor read FBackColor write SetBackColor;              // Background color
    property ForeColor: TColor read FForeColor write SetForeColor;              // Foreground color
    property PenMode: TPenMode read FPenMode write SetPenMode;                  // Pen write mode
    property PenStyle: TPenStyle read FPenStyle write SetPenStyle;              // Pen style
    property PenWidth: Integer read FPenWidth write SetPenWidth;                // Pen width
    property BrushStyle: TBrushStyle read FBrushStyle write SetBrushStyle;      // Brush style
    property OnClick;                                                           // Click event
    property OnDblClick;                                                        // Double click event
    procedure Paint; override;                                                  // Paint drawing
    constructor Create(AOwner: TComponent); override;                           // Create
    destructor Destroy; override;                                               // Destroy
  end;
procedure Register;                                                             // Component

implementation


//----------------------------------------------------------------------------------------------------------------------
// Create, destroy and set parameters


// Create and initialize
constructor TPinyoDrawing.Create(AOwner: TComponent);
begin
  inherited;
  PenMode:= pmCopy;                                                             // Copy mode
  PenStyle:= psSolid;                                                           // Solid line
  PenWidth:= 1;                                                                 // Thin
  BrushStyle:= bsSolid;                                                         // Solid background
  Points:= 17;                                                                  // Nice
  Picture:= pcStar;                                                             // Star
  BackColor:= clWhite;                                                          // White background
  ForeColor:= clBlack;                                                          // Black foreground
end;


// Release memory
destructor TPinyoDrawing.Destroy;
begin
  CirclePoints:= nil;                                                           // Release memory
  inherited;
end;


// Set number of points and allocate memory
procedure TPinyoDrawing.SetPoints(Value: Integer);
begin
  SetLength(CirclePoints,Value);                                                // Allocate memory
  FPoints:= Value;                                                              // Set
  Invalidate;                                                                   // Repaint
end;


// Set picture type
procedure TPinyoDrawing.SetPicture(Value: TPicture);
begin
  FPicture:= Value;                                                             // Set
  Invalidate;                                                                   // Repaint
end;


// Set background color
procedure TPinyoDrawing.SetBackColor(Value: TColor);
begin
  FBackColor:= Value;                                                           // Set
  Invalidate;                                                                   // Repaint
end;


// Set foreground color
procedure TPinyoDrawing.SetForeColor(Value: TColor);
begin
  FForeColor:= Value;                                                           // Set
  Invalidate;                                                                   // Repaint
end;


// Set pen mode
procedure TPinyoDrawing.SetPenMode(Value: TPenMode);
begin
  FPenMode:= Value;                                                             // Set
  Invalidate;                                                                   // Repaint
end;


// Set pen style
procedure TPinyoDrawing.SetPenStyle(Value: TPenStyle);
begin
  if Value<>psSolid then
  begin
    FPenWidth:= 1;                                                              // If it is not solid then width:=1
  end;
  FPenStyle:= Value;                                                            // Set
  Invalidate;                                                                   // Repaint
end;


// Set pen width
procedure TPinyoDrawing.SetPenWidth(Value: Integer);
begin
  if Value<>1 then
  begin
    FPenStyle:= psSolid;                                                        // If width<>1 then it must be solid
  end;
  FPenWidth:= Value;                                                            // Set
  Invalidate;                                                                   // Repaint
end;


// Set brush style
procedure TPinyoDrawing.SetBrushStyle(Value: TBrushStyle);
begin
  FBrushStyle:= Value;                                                          // Set
  Invalidate;                                                                   // Repaint
end;


//----------------------------------------------------------------------------------------------------------------------
// Calculate and draw


// Calculate point coordinates
procedure TPinyoDrawing.Calculate;
var
  i: Integer;
  d: Double;
begin
  d:=2*Pi/FPoints;                                                              // Distance
  for i:=0 to FPoints-1 do
  begin                                                                         // Points are on the circumference
    CirclePoints[i].x:=Center.x+Round(Radius*Sin(i*d));
    CirclePoints[i].y:=Center.y+Round(Radius*Cos(i*d));
  end;
end;


// Supress background paint
procedure TPinyoDrawing.WMEraseBkgnd(var msg: TWMEraseBkgnd);
begin
  msg.Result:= LRESULT(False);                                                  // Do it in Paint not now
end;


// Draw a picture
procedure TPinyoDrawing.Paint;
var
  i, j, k, r: Integer;
begin
  if (BrushStyle=bsSolid) or (BrushStyle=bsClear) then
  begin
    Canvas.Brush.Color:= BackColor                                              // Background color
  end
  else
  begin
    Canvas.Brush.Color:= BackColor xor $FFFFFF;                                 // Background color inverted
  end;
  Canvas.Brush.Style:= BrushStyle;                                              // Background style
  Canvas.Pen.Color:= ForeColor;                                                 // Line color
  Canvas.Pen.Mode:= PenMode;                                                    // Line mode
  Canvas.Pen.Style:= PenStyle;                                                  // Line style
  Canvas.Pen.Width:= PenWidth;                                                  // Line width

  case FPicture of

    pcStar:                                                                     // Draw a star
    begin                                                                       // Set Center and Radius
      Center.x:=Width div 2;
      Center.y:=Height div 2;
      if Center.x>Center.y then Radius:=Center.y else Radius:=Center.x;
      Radius:= Radius - PenWidth div 2 - 1;                                     // Substract pen width
      if Radius<0 then Radius:= 0;                                              // Final check
      Calculate;                                                                // Calculate points on circumference
      Canvas.FillRect(Canvas.ClipRect);                                         // Paint background
      for i:=1 to ((FPoints-1) div 2) do
      begin                                                                     // Draw picture
        for j:=0 to FPoints-1 do
        begin
          Canvas.MoveTo(CirclePoints[j].x,CirclePoints[j].y);
          k:= j+i;
          if k>=FPoints then k:=k-FPoints;
          Canvas.LineTo(CirclePoints[k].x,CirclePoints[k].y);
        end;
      end;
      if (FPoints and 1)=0 then
      begin                                                                     // Middle of an even star
        i:=FPoints div 2;
        for j:=0 to i-1 do
        begin
          Canvas.MoveTo(CirclePoints[j].x,CirclePoints[j].y);
          Canvas.LineTo(CirclePoints[j+i].x,CirclePoints[j+i].y);
        end;
      end;
    end;

    pcPascal:                                                                   // Draw a Pascal curve
    begin                                                                       // Set Center and Radius
      Center.x:=Width div 2;
      Center.y:=Height div 2;
      if Center.x>Center.y then Radius:=Center.y else Radius:=Center.x;
      Radius:= Radius - PenWidth div 2 - 1;                                     // Substract pen width
      if Radius<0 then Radius:= 0;                                              // Final check
      Calculate;                                                                // Calculate points on circumference
      Canvas.FillRect(Canvas.ClipRect);                                         // Paint background

      // Draw picture
      j:=2;
      for i:=1 to FPoints-1 do
      begin
        Canvas.MoveTo(CirclePoints[i].x,CirclePoints[i].y);
        Canvas.LineTo(CirclePoints[j].x,CirclePoints[j].y);
        j:=j+2;
        if (j>=FPoints) then j:=j-FPoints;
      end;
    end;

    pcNephroid:                                                                 // Draw a Nephroid
    begin                                                                       // Set Center and Radius
      Center.x:=Width div 2;
      Center.y:=Height div 2;
      Radius:=Center.x div 2;
      r:=MulDiv(Center.y,7,10);
      if Radius>r then Radius:=r;
      Radius:= Radius - PenWidth div 2 - 1;                                     // Substract pen width
      if Radius<0 then Radius:= 0;                                              // Final check
      Calculate;                                                                // Calculate points on circumference
      Canvas.FillRect(Canvas.ClipRect);                                         // Paint background
      for i:=1 to FPoints-1 do                                                  // Draw picture
      begin
        r:=CirclePoints[i].x-Center.x;
        Canvas.Brush.Style:=bsClear;
        Canvas.Ellipse(CirclePoints[i].x-r,CirclePoints[i].y-r,CirclePoints[i].x+r,CirclePoints[i].y+r);
      end;
    end;
  end;
end;


//----------------------------------------------------------------------------------------------------------------------
// Component

// Register component
procedure Register;
begin
  RegisterComponents('Propix',[TPinyoDrawing]);                                 // Register component
end;

end.

