// WStar Drawing program

//  PROPIX, Written by Pintr Gbor
// Szkesfehrvr, Krivnyi u. 15.
// H-8000, HUNGARY
// Tel: +36 30 9972445
// Fax: +36 22 304326
// Email: propix@mail.datatrans.hu
//        Pinter.Gabor@freemail.hu
// Web: www.propix.hu
//      www.datatrans.hu/propix

// Revisions:
//  V1.0  01/19/1997  First version written in C
//  V3.0  02/19/1998  Rewritten in Delphi 3
//  V5.0  08/25/2000  Recompiled with Delphi 5
//  V5.01 11/14/2000  More features, source reorganized
//  V5.02 11/30/2000  Transparent background added
//  V5.03 12/13/2000  PinyoDrawing is a component

unit MainUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs, Menus,
  AboutUnit, DrawUnit, PointsUnit, ExtCtrls;

type
  TMainForm = class(TForm)
    MainMenu: TMainMenu;
    Picture: TMenuItem;
    Properties: TMenuItem;
    About: TMenuItem;
    Star: TMenuItem;
    PascalCurve: TMenuItem;
    Nephroid: TMenuItem;
    Points: TMenuItem;
    Background: TMenuItem;
    Foreground: TMenuItem;
    PinyoDrawing: TPinyoDrawing;
    procedure AboutClick(Sender: TObject);
    procedure BackgroundClick(Sender: TObject);
    procedure ForegroundClick(Sender: TObject);
    procedure StarClick(Sender: TObject);
    procedure PascalCurveClick(Sender: TObject);
    procedure NephroidClick(Sender: TObject);
    procedure PointsClick(Sender: TObject);
    procedure PictureClick(Sender: TObject);
  private
    procedure WMEraseBkgnd(var msg: TWMEraseBkgnd); message WM_ERASEBKGND;      // Supress background paint
    procedure WMWindowPosChanging(var msg: TWMWindowPosMsg); message WM_WINDOWPOSCHANGING; // Force redraw
  end;


var
  MainForm: TMainForm;

implementation

uses BackgroundUnit, ForegroundUnit;

{$R *.DFM}

//----------------------------------------------------------------------------------------------------------------------
// Create, resize and paint

// Supress background paint
procedure TMainForm.WMEraseBkgnd(var msg: TWMEraseBkgnd);
begin
  msg.Result:= LRESULT(False);
end;


// About to move
procedure TMainForm.WMWindowPosChanging(var msg: TWMWindowPosMsg);
begin
  if Assigned(PinyoDrawing) then
  begin
    if PinyoDrawing.BrushStyle=bsClear then
    begin
      msg.WindowPos.flags:= msg.WindowPos.flags or SWP_NOCOPYBITS;              // Force redraw
    end;
  end;
  inherited;
end;


//----------------------------------------------------------------------------------------------------------------------
// Menu and controls


// About
procedure TMainForm.AboutClick(Sender: TObject);
var
  AboutForm: TAboutForm;
begin
  AboutForm:= nil;
  try
    AboutForm:= TAboutForm.Create(Application);
    AboutForm.ShowModal;
  finally
    AboutForm.Free;
  end;
end;


// Background
procedure TMainForm.BackgroundClick(Sender: TObject);
begin
  BackgroundForm.ShowModal;                                                     // Display dialog
end;


// Foreground
procedure TMainForm.ForegroundClick(Sender: TObject);
begin
  ForegroundForm.ShowModal;                                                     // Display dialog
end;


// Draw a star next time
procedure TMainForm.StarClick(Sender: TObject);
begin
  PinyoDrawing.Picture:= pcStar;
end;


// Draw a star next time
procedure TMainForm.PascalCurveClick(Sender: TObject);
begin
  PinyoDrawing.Picture:= pcPascal;
end;


// Draw a star next time
procedure TMainForm.NephroidClick(Sender: TObject);
begin
  PinyoDrawing.Picture:= pcNephroid;
end;


// Change number of points
procedure TMainForm.PointsClick(Sender: TObject);
var
  NumPointsForm: TNumPointsForm;
begin
  NumPointsForm:= nil;
  try
    NumPointsForm:= TNumPointsForm.Create(Application);
    NumPointsForm.ShowModal;
  finally
    NumPointsForm.Free;
  end;
end;


// Check radio buttons
procedure TMainForm.PictureClick(Sender: TObject);
begin
  case (PinyoDrawing.Picture) of
    pcStar:     Star.Checked:= True;
    pcPascal:   PascalCurve.Checked:= True;
    pcNephroid: Nephroid.Checked:= True;
  end;
end;

end.
